@@ -4,7 +4,7 @@
 
 use anyhow::Result;
 use wasmtime::*;
-use wasmtime_wasi::{sync::WasiCtxBuilder, Wasi};
+use wasmtime_wasi::sync::{Wasi, WasiCtxBuilder};
 
 fn main() -> Result<()> {
     let engine = Engine::default();
@@ -18,7 +18,7 @@ fn main() -> Result<()> {
         WasiCtxBuilder::new()
             .inherit_stdio()
             .inherit_args()?
-            .build()?,
+            .build(),
     );
     wasi.add_to_linker(&mut linker)?;
 
