@@ -5,7 +5,7 @@
 
 use anyhow::Result;
 use wasmtime::*;
-use wasmtime_wasi::{sync::WasiCtxBuilder, Wasi};
+use wasmtime_wasi::sync::{Wasi, WasiCtxBuilder};
 
 fn main() -> Result<()> {
     tracing_subscriber::FmtSubscriber::builder()
@@ -27,7 +27,7 @@ fn main() -> Result<()> {
         WasiCtxBuilder::new()
             .inherit_stdio()
             .inherit_args()?
-            .build()?
+            .build()
     )
     .is_ok());
 
