@@ -51,11 +51,11 @@ fn link_twice_bad() -> Result<()> {
 
     // memories
     let ty = MemoryType::new(Limits::new(1, None));
-    let memory = Memory::new(&store, ty);
+    let memory = Memory::new(&store, ty)?;
     linker.define("m", "", memory.clone())?;
     assert!(linker.define("m", "", memory.clone()).is_err());
     let ty = MemoryType::new(Limits::new(2, None));
-    let memory = Memory::new(&store, ty);
+    let memory = Memory::new(&store, ty)?;
     assert!(linker.define("m", "", memory.clone()).is_err());
 
     // tables
@@ -162,6 +162,24 @@ fn module_interposition() -> Result<()> {
     Ok(())
 }
 
+#[test]
+fn allow_unknown_exports() -> Result<()> {
+    let store = Store::default();
+    let mut linker = Linker::new(&store);
+    let module = Module::new(
+        store.engine(),
+        r#"(module (func (export "_start")) (global (export "g") i32 (i32.const 0)))"#,
+    )?;
+
+    assert!(linker.module("module", &module).is_err());
+
+    let mut linker = Linker::new(&store);
+    linker.allow_unknown_exports(true);
+    linker.module("module", &module)?;
+
+    Ok(())
+}
+
 #[test]
 fn no_leak() -> Result<()> {
     struct DropMe(Rc<Cell<bool>>);
